#!/usr/bin/env python3
"""
SSL/TLS Cipher Suite Tester
Parameters:
    - IP: Target IP address
    - port: Target port number (default: 443)
"""
import ssl
import socket
import argparse
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime

class SSLCipherTester:
    def __init__(self, ip, port=443):
        self.ip = ip
        self.port = port
        self.results = []
        
    def test_cipher(self, cipher):
        try:
            context = ssl.SSLContext(ssl.PROTOCOL_TLS)
            context.set_ciphers(cipher)
            
            with socket.create_connection((self.ip, self.port)) as sock:
                with context.wrap_socket(sock, server_hostname=self.ip) as ssock:
                    cipher_used = ssock.cipher()
                    return {
                        'cipher': cipher,
                        'status': 'Supported',
                        'details': cipher_used
                    }
        except ssl.SSLError:
            return {
                'cipher': cipher,
                'status': 'Not Supported',
                'details': None
            }
        except Exception as e:
            return {
                'cipher': cipher,
                'status': 'Error',
                'details': str(e)
            }
    
    def run_tests(self):
        print(f"Testing SSL/TLS cipher suites on {self.ip}:{self.port}")
        print("This may take a few minutes...\n")
        
        # Get all available ciphers
        context = ssl.SSLContext(ssl.PROTOCOL_TLS)
        ciphers = context.get_ciphers()
        
        # Test each cipher
        with ThreadPoolExecutor(max_workers=10) as executor:
            futures = []
            for cipher_info in ciphers:
                cipher = cipher_info['name']
                futures.append(executor.submit(self.test_cipher, cipher))
            
            # Process results
            supported = []
            unsupported = []
            errors = []
            
            for future in futures:
                result = future.result()
                if result['status'] == 'Supported':
                    supported.append(result)
                elif result['status'] == 'Not Supported':
                    unsupported.append(result)
                else:
                    errors.append(result)
        
        # Print results
        print(f"Test completed at {datetime.now()}\n")
        print(f"Total ciphers tested: {len(ciphers)}")
        print(f"Supported: {len(supported)}")
        print(f"Unsupported: {len(unsupported)}")
        print(f"Errors: {len(errors)}\n")
        
        if supported:
            print("Supported Ciphers:")
            print("-" * 60)
            for result in supported:
                cipher_details = result['details']
                print(f"Cipher: {result['cipher']}")
                print(f"Protocol: {cipher_details[1]}")
                print(f"Bits: {cipher_details[2]}")
                print("-" * 60)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="SSL/TLS Cipher Suite Tester")
    parser.add_argument("ip", help="Target IP address")
    parser.add_argument("--port", type=int, default=443,
                      help="Target port number (default: 443)")
    args = parser.parse_args()
    
    tester = SSLCipherTester(args.ip, args.port)
    tester.run_tests()
